package cokr.xit.fims.cmmn.dao;

import java.util.Map;

import org.egovframe.rte.psl.dataaccess.mapper.Mapper;

import cokr.xit.foundation.component.AbstractMapper;
import cokr.xit.foundation.data.DataObject;

@Mapper("stngMapper")
public interface StngMapper extends AbstractMapper {
	/**사용자의 기본 업무를 조회한다.
	 * @param userId 유저ID
	 * @return 기본업무
	 */
	DataObject selectDefaultTask(String userId);

	/**사용자 설정 정보를 조회한다.
	 * @param userId 사용자ID
	 * @return 사용자 설정정보
	 */
	DataObject selectUserStng(String userId);

	/**부서 설정 정보를 조회한다.
	 * @param deptCd 부서코드
	 * @return 부서 설정정보
	 */
	DataObject selectDeptStng(String deptCd);

	/**시군구 설정 정보를 조회한다.
	 * @param sggCd 시군구코드
	 * @return 시군구 설정정보
	 */
	DataObject selectSggStng(String sggCd);

	/**교부 계좌번호를 변경한다.
	 * @param params 교부계좌 변경 정보
	 * @return 저장여부
	 */
	int updateDtbnActno(Map<String, String> params);

	/**사용자의 기본 업무를 등록한다.
	 * @param params 사용자 설정 정보
	 * @return 저장여부
	 */
	int insertDefaultTask(Map<String, String> params);

	/**사용자의 기본 업무를 변경한다.
	 * @param params 사용자 설정 정보
	 * @return 저장여부
	 */
	int updateDefaultTask(Map<String, String> params);
}